#include "mcrypt_internal.h"

/* This is automatically created. Don't touch... */

extern void cbc_LTX__init_mcrypt(void);
extern void cbc_LTX__mcrypt_set_state(void);
extern void cbc_LTX__mcrypt_get_state(void);
extern void cbc_LTX__end_mcrypt(void);
extern void cbc_LTX__mcrypt(void);
extern void cbc_LTX__mdecrypt(void);
extern void cbc_LTX__has_iv(void);
extern void cbc_LTX__is_block_mode(void);
extern void cbc_LTX__is_block_algorithm_mode(void);
extern void cbc_LTX__mcrypt_get_modes_name(void);
extern void cbc_LTX__mcrypt_mode_get_size(void);
extern void cbc_LTX__mcrypt_mode_version(void);
extern void cfb_LTX__init_mcrypt(void);
extern void cfb_LTX__mcrypt_set_state(void);
extern void cfb_LTX__mcrypt_get_state(void);
extern void cfb_LTX__end_mcrypt(void);
extern void cfb_LTX__mcrypt(void);
extern void cfb_LTX__mdecrypt(void);
extern void cfb_LTX__has_iv(void);
extern void cfb_LTX__is_block_mode(void);
extern void cfb_LTX__is_block_algorithm_mode(void);
extern void cfb_LTX__mcrypt_get_modes_name(void);
extern void cfb_LTX__mcrypt_mode_get_size(void);
extern void cfb_LTX__mcrypt_mode_version(void);
extern void ctr_LTX__init_mcrypt(void);
extern void ctr_LTX__mcrypt_set_state(void);
extern void ctr_LTX__mcrypt_get_state(void);
extern void ctr_LTX__end_mcrypt(void);
extern void ctr_LTX__mcrypt(void);
extern void ctr_LTX__mdecrypt(void);
extern void ctr_LTX__has_iv(void);
extern void ctr_LTX__is_block_mode(void);
extern void ctr_LTX__is_block_algorithm_mode(void);
extern void ctr_LTX__mcrypt_get_modes_name(void);
extern void ctr_LTX__mcrypt_mode_get_size(void);
extern void ctr_LTX__mcrypt_mode_version(void);
extern void ecb_LTX__mcrypt_set_state(void);
extern void ecb_LTX__mcrypt_get_state(void);
extern void ecb_LTX__init_mcrypt(void);
extern void ecb_LTX__end_mcrypt(void);
extern void ecb_LTX__mcrypt(void);
extern void ecb_LTX__mdecrypt(void);
extern void ecb_LTX__has_iv(void);
extern void ecb_LTX__is_block_mode(void);
extern void ecb_LTX__is_block_algorithm_mode(void);
extern void ecb_LTX__mcrypt_get_modes_name(void);
extern void ecb_LTX__mcrypt_mode_get_size(void);
extern void ecb_LTX__mcrypt_mode_version(void);
extern void ncfb_LTX__init_mcrypt(void);
extern void ncfb_LTX__mcrypt_set_state(void);
extern void ncfb_LTX__mcrypt_get_state(void);
extern void ncfb_LTX__end_mcrypt(void);
extern void ncfb_LTX__mcrypt(void);
extern void ncfb_LTX__mdecrypt(void);
extern void ncfb_LTX__has_iv(void);
extern void ncfb_LTX__is_block_mode(void);
extern void ncfb_LTX__is_block_algorithm_mode(void);
extern void ncfb_LTX__mcrypt_get_modes_name(void);
extern void ncfb_LTX__mcrypt_mode_get_size(void);
extern void ncfb_LTX__mcrypt_mode_version(void);
extern void nofb_LTX__init_mcrypt(void);
extern void nofb_LTX__mcrypt_set_state(void);
extern void nofb_LTX__mcrypt_get_state(void);
extern void nofb_LTX__end_mcrypt(void);
extern void nofb_LTX__mcrypt(void);
extern void nofb_LTX__mdecrypt(void);
extern void nofb_LTX__has_iv(void);
extern void nofb_LTX__is_block_mode(void);
extern void nofb_LTX__is_block_algorithm_mode(void);
extern void nofb_LTX__mcrypt_get_modes_name(void);
extern void nofb_LTX__mcrypt_mode_get_size(void);
extern void nofb_LTX__mcrypt_mode_version(void);
extern void ofb_LTX__init_mcrypt(void);
extern void ofb_LTX__mcrypt_set_state(void);
extern void ofb_LTX__mcrypt_get_state(void);
extern void ofb_LTX__end_mcrypt(void);
extern void ofb_LTX__mcrypt(void);
extern void ofb_LTX__mdecrypt(void);
extern void ofb_LTX__has_iv(void);
extern void ofb_LTX__is_block_mode(void);
extern void ofb_LTX__is_block_algorithm_mode(void);
extern void ofb_LTX__mcrypt_get_modes_name(void);
extern void ofb_LTX__mcrypt_mode_get_size(void);
extern void ofb_LTX__mcrypt_mode_version(void);
extern void stream_LTX__init_mcrypt(void);
extern void stream_LTX__mcrypt_set_state(void);
extern void stream_LTX__mcrypt_get_state(void);
extern void stream_LTX__end_mcrypt(void);
extern void stream_LTX__mcrypt(void);
extern void stream_LTX__mdecrypt(void);
extern void stream_LTX__has_iv(void);
extern void stream_LTX__is_block_mode(void);
extern void stream_LTX__is_block_algorithm_mode(void);
extern void stream_LTX__mcrypt_get_modes_name(void);
extern void stream_LTX__mcrypt_mode_get_size(void);
extern void stream_LTX__mcrypt_mode_version(void);
extern void cast_128_LTX__mcrypt_set_key(void);
extern void cast_128_LTX__mcrypt_encrypt(void);
extern void cast_128_LTX__mcrypt_decrypt(void);
extern void cast_128_LTX__mcrypt_get_size(void);
extern void cast_128_LTX__mcrypt_get_block_size(void);
extern void cast_128_LTX__is_block_algorithm(void);
extern void cast_128_LTX__mcrypt_get_key_size(void);
extern void cast_128_LTX__mcrypt_get_supported_key_sizes(void);
extern void cast_128_LTX__mcrypt_get_algorithms_name(void);
extern void cast_128_LTX__mcrypt_self_test(void);
extern void cast_128_LTX__mcrypt_algorithm_version(void);
extern void gost_LTX__mcrypt_set_key(void);
extern void gost_LTX__mcrypt_encrypt(void);
extern void gost_LTX__mcrypt_decrypt(void);
extern void gost_LTX__mcrypt_get_size(void);
extern void gost_LTX__mcrypt_get_block_size(void);
extern void gost_LTX__is_block_algorithm(void);
extern void gost_LTX__mcrypt_get_key_size(void);
extern void gost_LTX__mcrypt_get_supported_key_sizes(void);
extern void gost_LTX__mcrypt_get_algorithms_name(void);
extern void gost_LTX__mcrypt_self_test(void);
extern void gost_LTX__mcrypt_algorithm_version(void);
extern void rijndael_128_LTX__mcrypt_set_key(void);
extern void rijndael_128_LTX__mcrypt_encrypt(void);
extern void rijndael_128_LTX__mcrypt_decrypt(void);
extern void rijndael_128_LTX__mcrypt_get_size(void);
extern void rijndael_128_LTX__mcrypt_get_block_size(void);
extern void rijndael_128_LTX__is_block_algorithm(void);
extern void rijndael_128_LTX__mcrypt_get_key_size(void);
extern void rijndael_128_LTX__mcrypt_get_supported_key_sizes(void);
extern void rijndael_128_LTX__mcrypt_get_algorithms_name(void);
extern void rijndael_128_LTX__mcrypt_self_test(void);
extern void rijndael_128_LTX__mcrypt_algorithm_version(void);
extern void twofish_LTX__mcrypt_set_key(void);
extern void twofish_LTX__mcrypt_encrypt(void);
extern void twofish_LTX__mcrypt_decrypt(void);
extern void twofish_LTX__mcrypt_get_size(void);
extern void twofish_LTX__mcrypt_get_block_size(void);
extern void twofish_LTX__is_block_algorithm(void);
extern void twofish_LTX__mcrypt_get_key_size(void);
extern void twofish_LTX__mcrypt_get_supported_key_sizes(void);
extern void twofish_LTX__mcrypt_get_algorithms_name(void);
extern void twofish_LTX__mcrypt_self_test(void);
extern void twofish_LTX__mcrypt_algorithm_version(void);
extern void arcfour_LTX__mcrypt_set_key(void);
extern void arcfour_LTX__mcrypt_encrypt(void);
extern void arcfour_LTX__mcrypt_decrypt(void);
extern void arcfour_LTX__mcrypt_get_size(void);
extern void arcfour_LTX__mcrypt_get_block_size(void);
extern void arcfour_LTX__is_block_algorithm(void);
extern void arcfour_LTX__mcrypt_get_key_size(void);
extern void arcfour_LTX__mcrypt_get_algo_iv_size(void);
extern void arcfour_LTX__mcrypt_get_supported_key_sizes(void);
extern void arcfour_LTX__mcrypt_get_algorithms_name(void);
extern void arcfour_LTX__mcrypt_self_test(void);
extern void arcfour_LTX__mcrypt_algorithm_version(void);
extern void cast_256_LTX__mcrypt_set_key(void);
extern void cast_256_LTX__mcrypt_encrypt(void);
extern void cast_256_LTX__mcrypt_decrypt(void);
extern void cast_256_LTX__mcrypt_get_size(void);
extern void cast_256_LTX__mcrypt_get_block_size(void);
extern void cast_256_LTX__is_block_algorithm(void);
extern void cast_256_LTX__mcrypt_get_key_size(void);
extern void cast_256_LTX__mcrypt_get_supported_key_sizes(void);
extern void cast_256_LTX__mcrypt_get_algorithms_name(void);
extern void cast_256_LTX__mcrypt_self_test(void);
extern void cast_256_LTX__mcrypt_algorithm_version(void);
extern void loki97_LTX__mcrypt_set_key(void);
extern void loki97_LTX__mcrypt_encrypt(void);
extern void loki97_LTX__mcrypt_decrypt(void);
extern void loki97_LTX__mcrypt_get_size(void);
extern void loki97_LTX__mcrypt_get_block_size(void);
extern void loki97_LTX__is_block_algorithm(void);
extern void loki97_LTX__mcrypt_get_key_size(void);
extern void loki97_LTX__mcrypt_get_supported_key_sizes(void);
extern void loki97_LTX__mcrypt_get_algorithms_name(void);
extern void loki97_LTX__mcrypt_self_test(void);
extern void loki97_LTX__mcrypt_algorithm_version(void);
extern void rijndael_192_LTX__mcrypt_set_key(void);
extern void rijndael_192_LTX__mcrypt_encrypt(void);
extern void rijndael_192_LTX__mcrypt_decrypt(void);
extern void rijndael_192_LTX__mcrypt_get_size(void);
extern void rijndael_192_LTX__mcrypt_get_block_size(void);
extern void rijndael_192_LTX__is_block_algorithm(void);
extern void rijndael_192_LTX__mcrypt_get_key_size(void);
extern void rijndael_192_LTX__mcrypt_get_supported_key_sizes(void);
extern void rijndael_192_LTX__mcrypt_get_algorithms_name(void);
extern void rijndael_192_LTX__mcrypt_self_test(void);
extern void rijndael_192_LTX__mcrypt_algorithm_version(void);
extern void saferplus_LTX__mcrypt_set_key(void);
extern void saferplus_LTX__mcrypt_encrypt(void);
extern void saferplus_LTX__mcrypt_decrypt(void);
extern void saferplus_LTX__mcrypt_get_size(void);
extern void saferplus_LTX__mcrypt_get_block_size(void);
extern void saferplus_LTX__is_block_algorithm(void);
extern void saferplus_LTX__mcrypt_get_key_size(void);
extern void saferplus_LTX__mcrypt_get_supported_key_sizes(void);
extern void saferplus_LTX__mcrypt_get_algorithms_name(void);
extern void saferplus_LTX__mcrypt_self_test(void);
extern void saferplus_LTX__mcrypt_algorithm_version(void);
extern void wake_LTX__mcrypt_set_key(void);
extern void wake_LTX__mcrypt_encrypt(void);
extern void wake_LTX__mcrypt_decrypt(void);
extern void wake_LTX__mcrypt_get_size(void);
extern void wake_LTX__mcrypt_get_block_size(void);
extern void wake_LTX__is_block_algorithm(void);
extern void wake_LTX__mcrypt_get_key_size(void);
extern void wake_LTX__mcrypt_get_algo_iv_size(void);
extern void wake_LTX__mcrypt_get_supported_key_sizes(void);
extern void wake_LTX__mcrypt_get_algorithms_name(void);
extern void wake_LTX__mcrypt_self_test(void);
extern void wake_LTX__mcrypt_algorithm_version(void);
extern void blowfish_compat_LTX__mcrypt_set_key(void);
extern void blowfish_compat_LTX__mcrypt_encrypt(void);
extern void blowfish_compat_LTX__mcrypt_decrypt(void);
extern void blowfish_compat_LTX__mcrypt_get_size(void);
extern void blowfish_compat_LTX__mcrypt_get_block_size(void);
extern void blowfish_compat_LTX__is_block_algorithm(void);
extern void blowfish_compat_LTX__mcrypt_get_key_size(void);
extern void blowfish_compat_LTX__mcrypt_get_supported_key_sizes(void);
extern void blowfish_compat_LTX__mcrypt_get_algorithms_name(void);
extern void blowfish_compat_LTX__mcrypt_self_test(void);
extern void blowfish_compat_LTX__mcrypt_algorithm_version(void);
extern void des_LTX__mcrypt_set_key(void);
extern void des_LTX__mcrypt_encrypt(void);
extern void des_LTX__mcrypt_decrypt(void);
extern void des_LTX__mcrypt_get_size(void);
extern void des_LTX__mcrypt_get_block_size(void);
extern void des_LTX__is_block_algorithm(void);
extern void des_LTX__mcrypt_get_key_size(void);
extern void des_LTX__mcrypt_get_supported_key_sizes(void);
extern void des_LTX__mcrypt_get_algorithms_name(void);
extern void des_LTX__mcrypt_self_test(void);
extern void des_LTX__mcrypt_algorithm_version(void);
extern void rijndael_256_LTX__mcrypt_set_key(void);
extern void rijndael_256_LTX__mcrypt_encrypt(void);
extern void rijndael_256_LTX__mcrypt_decrypt(void);
extern void rijndael_256_LTX__mcrypt_get_size(void);
extern void rijndael_256_LTX__mcrypt_get_block_size(void);
extern void rijndael_256_LTX__is_block_algorithm(void);
extern void rijndael_256_LTX__mcrypt_get_key_size(void);
extern void rijndael_256_LTX__mcrypt_get_supported_key_sizes(void);
extern void rijndael_256_LTX__mcrypt_get_algorithms_name(void);
extern void rijndael_256_LTX__mcrypt_self_test(void);
extern void rijndael_256_LTX__mcrypt_algorithm_version(void);
extern void serpent_LTX__mcrypt_set_key(void);
extern void serpent_LTX__mcrypt_encrypt(void);
extern void serpent_LTX__mcrypt_decrypt(void);
extern void serpent_LTX__mcrypt_get_size(void);
extern void serpent_LTX__mcrypt_get_block_size(void);
extern void serpent_LTX__is_block_algorithm(void);
extern void serpent_LTX__mcrypt_get_key_size(void);
extern void serpent_LTX__mcrypt_get_supported_key_sizes(void);
extern void serpent_LTX__mcrypt_get_algorithms_name(void);
extern void serpent_LTX__mcrypt_self_test(void);
extern void serpent_LTX__mcrypt_algorithm_version(void);
extern void xtea_LTX__mcrypt_set_key(void);
extern void xtea_LTX__mcrypt_encrypt(void);
extern void xtea_LTX__mcrypt_decrypt(void);
extern void xtea_LTX__mcrypt_get_size(void);
extern void xtea_LTX__mcrypt_get_block_size(void);
extern void xtea_LTX__is_block_algorithm(void);
extern void xtea_LTX__mcrypt_get_key_size(void);
extern void xtea_LTX__mcrypt_get_supported_key_sizes(void);
extern void xtea_LTX__mcrypt_get_algorithms_name(void);
extern void xtea_LTX__mcrypt_self_test(void);
extern void xtea_LTX__mcrypt_algorithm_version(void);
extern void blowfish_LTX__mcrypt_set_key(void);
extern void blowfish_LTX__mcrypt_encrypt(void);
extern void blowfish_LTX__mcrypt_decrypt(void);
extern void blowfish_LTX__mcrypt_get_size(void);
extern void blowfish_LTX__mcrypt_get_block_size(void);
extern void blowfish_LTX__is_block_algorithm(void);
extern void blowfish_LTX__mcrypt_get_key_size(void);
extern void blowfish_LTX__mcrypt_get_supported_key_sizes(void);
extern void blowfish_LTX__mcrypt_get_algorithms_name(void);
extern void blowfish_LTX__mcrypt_self_test(void);
extern void blowfish_LTX__mcrypt_algorithm_version(void);
extern void enigma_LTX__mcrypt_set_key(void);
extern void enigma_LTX__mcrypt_encrypt(void);
extern void enigma_LTX__mcrypt_decrypt(void);
extern void enigma_LTX__mcrypt_get_size(void);
extern void enigma_LTX__mcrypt_get_block_size(void);
extern void enigma_LTX__is_block_algorithm(void);
extern void enigma_LTX__mcrypt_get_key_size(void);
extern void enigma_LTX__mcrypt_get_algo_iv_size(void);
extern void enigma_LTX__mcrypt_get_supported_key_sizes(void);
extern void enigma_LTX__mcrypt_get_algorithms_name(void);
extern void enigma_LTX__mcrypt_self_test(void);
extern void enigma_LTX__mcrypt_algorithm_version(void);
extern void rc2_LTX__mcrypt_set_key(void);
extern void rc2_LTX__mcrypt_encrypt(void);
extern void rc2_LTX__mcrypt_decrypt(void);
extern void rc2_LTX__mcrypt_get_size(void);
extern void rc2_LTX__mcrypt_get_block_size(void);
extern void rc2_LTX__is_block_algorithm(void);
extern void rc2_LTX__mcrypt_get_key_size(void);
extern void rc2_LTX__mcrypt_get_supported_key_sizes(void);
extern void rc2_LTX__mcrypt_get_algorithms_name(void);
extern void rc2_LTX__mcrypt_self_test(void);
extern void rc2_LTX__mcrypt_algorithm_version(void);
extern void tripledes_LTX__mcrypt_set_key(void);
extern void tripledes_LTX__mcrypt_encrypt(void);
extern void tripledes_LTX__mcrypt_decrypt(void);
extern void tripledes_LTX__mcrypt_get_size(void);
extern void tripledes_LTX__mcrypt_get_block_size(void);
extern void tripledes_LTX__is_block_algorithm(void);
extern void tripledes_LTX__mcrypt_get_key_size(void);
extern void tripledes_LTX__mcrypt_get_supported_key_sizes(void);
extern void tripledes_LTX__mcrypt_get_algorithms_name(void);
extern void tripledes_LTX__mcrypt_self_test(void);
extern void tripledes_LTX__mcrypt_algorithm_version(void);

const mcrypt_preloaded mps[] = {
	{"cbc", NULL}, 
	{"cbc_LTX__init_mcrypt", cbc_LTX__init_mcrypt},
	{"cbc_LTX__mcrypt_set_state", cbc_LTX__mcrypt_set_state},
	{"cbc_LTX__mcrypt_get_state", cbc_LTX__mcrypt_get_state},
	{"cbc_LTX__end_mcrypt", cbc_LTX__end_mcrypt},
	{"cbc_LTX__mcrypt", cbc_LTX__mcrypt},
	{"cbc_LTX__mdecrypt", cbc_LTX__mdecrypt},
	{"cbc_LTX__has_iv", cbc_LTX__has_iv},
	{"cbc_LTX__is_block_mode", cbc_LTX__is_block_mode},
	{"cbc_LTX__is_block_algorithm_mode", cbc_LTX__is_block_algorithm_mode},
	{"cbc_LTX__mcrypt_get_modes_name", cbc_LTX__mcrypt_get_modes_name},
	{"cbc_LTX__mcrypt_mode_get_size", cbc_LTX__mcrypt_mode_get_size},
	{"cbc_LTX__mcrypt_mode_version", cbc_LTX__mcrypt_mode_version},
	{"cfb", NULL}, 
	{"cfb_LTX__init_mcrypt", cfb_LTX__init_mcrypt},
	{"cfb_LTX__mcrypt_set_state", cfb_LTX__mcrypt_set_state},
	{"cfb_LTX__mcrypt_get_state", cfb_LTX__mcrypt_get_state},
	{"cfb_LTX__end_mcrypt", cfb_LTX__end_mcrypt},
	{"cfb_LTX__mcrypt", cfb_LTX__mcrypt},
	{"cfb_LTX__mdecrypt", cfb_LTX__mdecrypt},
	{"cfb_LTX__has_iv", cfb_LTX__has_iv},
	{"cfb_LTX__is_block_mode", cfb_LTX__is_block_mode},
	{"cfb_LTX__is_block_algorithm_mode", cfb_LTX__is_block_algorithm_mode},
	{"cfb_LTX__mcrypt_get_modes_name", cfb_LTX__mcrypt_get_modes_name},
	{"cfb_LTX__mcrypt_mode_get_size", cfb_LTX__mcrypt_mode_get_size},
	{"cfb_LTX__mcrypt_mode_version", cfb_LTX__mcrypt_mode_version},
	{"ctr", NULL}, 
	{"ctr_LTX__init_mcrypt", ctr_LTX__init_mcrypt},
	{"ctr_LTX__mcrypt_set_state", ctr_LTX__mcrypt_set_state},
	{"ctr_LTX__mcrypt_get_state", ctr_LTX__mcrypt_get_state},
	{"ctr_LTX__end_mcrypt", ctr_LTX__end_mcrypt},
	{"ctr_LTX__mcrypt", ctr_LTX__mcrypt},
	{"ctr_LTX__mdecrypt", ctr_LTX__mdecrypt},
	{"ctr_LTX__has_iv", ctr_LTX__has_iv},
	{"ctr_LTX__is_block_mode", ctr_LTX__is_block_mode},
	{"ctr_LTX__is_block_algorithm_mode", ctr_LTX__is_block_algorithm_mode},
	{"ctr_LTX__mcrypt_get_modes_name", ctr_LTX__mcrypt_get_modes_name},
	{"ctr_LTX__mcrypt_mode_get_size", ctr_LTX__mcrypt_mode_get_size},
	{"ctr_LTX__mcrypt_mode_version", ctr_LTX__mcrypt_mode_version},
	{"ecb", NULL}, 
	{"ecb_LTX__mcrypt_set_state", ecb_LTX__mcrypt_set_state},
	{"ecb_LTX__mcrypt_get_state", ecb_LTX__mcrypt_get_state},
	{"ecb_LTX__init_mcrypt", ecb_LTX__init_mcrypt},
	{"ecb_LTX__end_mcrypt", ecb_LTX__end_mcrypt},
	{"ecb_LTX__mcrypt", ecb_LTX__mcrypt},
	{"ecb_LTX__mdecrypt", ecb_LTX__mdecrypt},
	{"ecb_LTX__has_iv", ecb_LTX__has_iv},
	{"ecb_LTX__is_block_mode", ecb_LTX__is_block_mode},
	{"ecb_LTX__is_block_algorithm_mode", ecb_LTX__is_block_algorithm_mode},
	{"ecb_LTX__mcrypt_get_modes_name", ecb_LTX__mcrypt_get_modes_name},
	{"ecb_LTX__mcrypt_mode_get_size", ecb_LTX__mcrypt_mode_get_size},
	{"ecb_LTX__mcrypt_mode_version", ecb_LTX__mcrypt_mode_version},
	{"ncfb", NULL}, 
	{"ncfb_LTX__init_mcrypt", ncfb_LTX__init_mcrypt},
	{"ncfb_LTX__mcrypt_set_state", ncfb_LTX__mcrypt_set_state},
	{"ncfb_LTX__mcrypt_get_state", ncfb_LTX__mcrypt_get_state},
	{"ncfb_LTX__end_mcrypt", ncfb_LTX__end_mcrypt},
	{"ncfb_LTX__mcrypt", ncfb_LTX__mcrypt},
	{"ncfb_LTX__mdecrypt", ncfb_LTX__mdecrypt},
	{"ncfb_LTX__has_iv", ncfb_LTX__has_iv},
	{"ncfb_LTX__is_block_mode", ncfb_LTX__is_block_mode},
	{"ncfb_LTX__is_block_algorithm_mode", ncfb_LTX__is_block_algorithm_mode},
	{"ncfb_LTX__mcrypt_get_modes_name", ncfb_LTX__mcrypt_get_modes_name},
	{"ncfb_LTX__mcrypt_mode_get_size", ncfb_LTX__mcrypt_mode_get_size},
	{"ncfb_LTX__mcrypt_mode_version", ncfb_LTX__mcrypt_mode_version},
	{"nofb", NULL}, 
	{"nofb_LTX__init_mcrypt", nofb_LTX__init_mcrypt},
	{"nofb_LTX__mcrypt_set_state", nofb_LTX__mcrypt_set_state},
	{"nofb_LTX__mcrypt_get_state", nofb_LTX__mcrypt_get_state},
	{"nofb_LTX__end_mcrypt", nofb_LTX__end_mcrypt},
	{"nofb_LTX__mcrypt", nofb_LTX__mcrypt},
	{"nofb_LTX__mdecrypt", nofb_LTX__mdecrypt},
	{"nofb_LTX__has_iv", nofb_LTX__has_iv},
	{"nofb_LTX__is_block_mode", nofb_LTX__is_block_mode},
	{"nofb_LTX__is_block_algorithm_mode", nofb_LTX__is_block_algorithm_mode},
	{"nofb_LTX__mcrypt_get_modes_name", nofb_LTX__mcrypt_get_modes_name},
	{"nofb_LTX__mcrypt_mode_get_size", nofb_LTX__mcrypt_mode_get_size},
	{"nofb_LTX__mcrypt_mode_version", nofb_LTX__mcrypt_mode_version},
	{"ofb", NULL}, 
	{"ofb_LTX__init_mcrypt", ofb_LTX__init_mcrypt},
	{"ofb_LTX__mcrypt_set_state", ofb_LTX__mcrypt_set_state},
	{"ofb_LTX__mcrypt_get_state", ofb_LTX__mcrypt_get_state},
	{"ofb_LTX__end_mcrypt", ofb_LTX__end_mcrypt},
	{"ofb_LTX__mcrypt", ofb_LTX__mcrypt},
	{"ofb_LTX__mdecrypt", ofb_LTX__mdecrypt},
	{"ofb_LTX__has_iv", ofb_LTX__has_iv},
	{"ofb_LTX__is_block_mode", ofb_LTX__is_block_mode},
	{"ofb_LTX__is_block_algorithm_mode", ofb_LTX__is_block_algorithm_mode},
	{"ofb_LTX__mcrypt_get_modes_name", ofb_LTX__mcrypt_get_modes_name},
	{"ofb_LTX__mcrypt_mode_get_size", ofb_LTX__mcrypt_mode_get_size},
	{"ofb_LTX__mcrypt_mode_version", ofb_LTX__mcrypt_mode_version},
	{"stream", NULL}, 
	{"stream_LTX__init_mcrypt", stream_LTX__init_mcrypt},
	{"stream_LTX__mcrypt_set_state", stream_LTX__mcrypt_set_state},
	{"stream_LTX__mcrypt_get_state", stream_LTX__mcrypt_get_state},
	{"stream_LTX__end_mcrypt", stream_LTX__end_mcrypt},
	{"stream_LTX__mcrypt", stream_LTX__mcrypt},
	{"stream_LTX__mdecrypt", stream_LTX__mdecrypt},
	{"stream_LTX__has_iv", stream_LTX__has_iv},
	{"stream_LTX__is_block_mode", stream_LTX__is_block_mode},
	{"stream_LTX__is_block_algorithm_mode", stream_LTX__is_block_algorithm_mode},
	{"stream_LTX__mcrypt_get_modes_name", stream_LTX__mcrypt_get_modes_name},
	{"stream_LTX__mcrypt_mode_get_size", stream_LTX__mcrypt_mode_get_size},
	{"stream_LTX__mcrypt_mode_version", stream_LTX__mcrypt_mode_version},
	{"cast-128", NULL}, 
	{"cast_128_LTX__mcrypt_set_key", cast_128_LTX__mcrypt_set_key},
	{"cast_128_LTX__mcrypt_encrypt", cast_128_LTX__mcrypt_encrypt},
	{"cast_128_LTX__mcrypt_decrypt", cast_128_LTX__mcrypt_decrypt},
	{"cast_128_LTX__mcrypt_get_size", cast_128_LTX__mcrypt_get_size},
	{"cast_128_LTX__mcrypt_get_block_size", cast_128_LTX__mcrypt_get_block_size},
	{"cast_128_LTX__is_block_algorithm", cast_128_LTX__is_block_algorithm},
	{"cast_128_LTX__mcrypt_get_key_size", cast_128_LTX__mcrypt_get_key_size},
	{"cast_128_LTX__mcrypt_get_supported_key_sizes", cast_128_LTX__mcrypt_get_supported_key_sizes},
	{"cast_128_LTX__mcrypt_get_algorithms_name", cast_128_LTX__mcrypt_get_algorithms_name},
	{"cast_128_LTX__mcrypt_self_test", cast_128_LTX__mcrypt_self_test},
	{"cast_128_LTX__mcrypt_algorithm_version", cast_128_LTX__mcrypt_algorithm_version},
	{"gost", NULL}, 
	{"gost_LTX__mcrypt_set_key", gost_LTX__mcrypt_set_key},
	{"gost_LTX__mcrypt_encrypt", gost_LTX__mcrypt_encrypt},
	{"gost_LTX__mcrypt_decrypt", gost_LTX__mcrypt_decrypt},
	{"gost_LTX__mcrypt_get_size", gost_LTX__mcrypt_get_size},
	{"gost_LTX__mcrypt_get_block_size", gost_LTX__mcrypt_get_block_size},
	{"gost_LTX__is_block_algorithm", gost_LTX__is_block_algorithm},
	{"gost_LTX__mcrypt_get_key_size", gost_LTX__mcrypt_get_key_size},
	{"gost_LTX__mcrypt_get_supported_key_sizes", gost_LTX__mcrypt_get_supported_key_sizes},
	{"gost_LTX__mcrypt_get_algorithms_name", gost_LTX__mcrypt_get_algorithms_name},
	{"gost_LTX__mcrypt_self_test", gost_LTX__mcrypt_self_test},
	{"gost_LTX__mcrypt_algorithm_version", gost_LTX__mcrypt_algorithm_version},
	{"rijndael-128", NULL}, 
	{"rijndael_128_LTX__mcrypt_set_key", rijndael_128_LTX__mcrypt_set_key},
	{"rijndael_128_LTX__mcrypt_encrypt", rijndael_128_LTX__mcrypt_encrypt},
	{"rijndael_128_LTX__mcrypt_decrypt", rijndael_128_LTX__mcrypt_decrypt},
	{"rijndael_128_LTX__mcrypt_get_size", rijndael_128_LTX__mcrypt_get_size},
	{"rijndael_128_LTX__mcrypt_get_block_size", rijndael_128_LTX__mcrypt_get_block_size},
	{"rijndael_128_LTX__is_block_algorithm", rijndael_128_LTX__is_block_algorithm},
	{"rijndael_128_LTX__mcrypt_get_key_size", rijndael_128_LTX__mcrypt_get_key_size},
	{"rijndael_128_LTX__mcrypt_get_supported_key_sizes", rijndael_128_LTX__mcrypt_get_supported_key_sizes},
	{"rijndael_128_LTX__mcrypt_get_algorithms_name", rijndael_128_LTX__mcrypt_get_algorithms_name},
	{"rijndael_128_LTX__mcrypt_self_test", rijndael_128_LTX__mcrypt_self_test},
	{"rijndael_128_LTX__mcrypt_algorithm_version", rijndael_128_LTX__mcrypt_algorithm_version},
	{"twofish", NULL}, 
	{"twofish_LTX__mcrypt_set_key", twofish_LTX__mcrypt_set_key},
	{"twofish_LTX__mcrypt_encrypt", twofish_LTX__mcrypt_encrypt},
	{"twofish_LTX__mcrypt_decrypt", twofish_LTX__mcrypt_decrypt},
	{"twofish_LTX__mcrypt_get_size", twofish_LTX__mcrypt_get_size},
	{"twofish_LTX__mcrypt_get_block_size", twofish_LTX__mcrypt_get_block_size},
	{"twofish_LTX__is_block_algorithm", twofish_LTX__is_block_algorithm},
	{"twofish_LTX__mcrypt_get_key_size", twofish_LTX__mcrypt_get_key_size},
	{"twofish_LTX__mcrypt_get_supported_key_sizes", twofish_LTX__mcrypt_get_supported_key_sizes},
	{"twofish_LTX__mcrypt_get_algorithms_name", twofish_LTX__mcrypt_get_algorithms_name},
	{"twofish_LTX__mcrypt_self_test", twofish_LTX__mcrypt_self_test},
	{"twofish_LTX__mcrypt_algorithm_version", twofish_LTX__mcrypt_algorithm_version},
	{"arcfour", NULL}, 
	{"arcfour_LTX__mcrypt_set_key", arcfour_LTX__mcrypt_set_key},
	{"arcfour_LTX__mcrypt_encrypt", arcfour_LTX__mcrypt_encrypt},
	{"arcfour_LTX__mcrypt_decrypt", arcfour_LTX__mcrypt_decrypt},
	{"arcfour_LTX__mcrypt_get_size", arcfour_LTX__mcrypt_get_size},
	{"arcfour_LTX__mcrypt_get_block_size", arcfour_LTX__mcrypt_get_block_size},
	{"arcfour_LTX__is_block_algorithm", arcfour_LTX__is_block_algorithm},
	{"arcfour_LTX__mcrypt_get_key_size", arcfour_LTX__mcrypt_get_key_size},
	{"arcfour_LTX__mcrypt_get_algo_iv_size", arcfour_LTX__mcrypt_get_algo_iv_size},
	{"arcfour_LTX__mcrypt_get_supported_key_sizes", arcfour_LTX__mcrypt_get_supported_key_sizes},
	{"arcfour_LTX__mcrypt_get_algorithms_name", arcfour_LTX__mcrypt_get_algorithms_name},
	{"arcfour_LTX__mcrypt_self_test", arcfour_LTX__mcrypt_self_test},
	{"arcfour_LTX__mcrypt_algorithm_version", arcfour_LTX__mcrypt_algorithm_version},
	{"cast-256", NULL}, 
	{"cast_256_LTX__mcrypt_set_key", cast_256_LTX__mcrypt_set_key},
	{"cast_256_LTX__mcrypt_encrypt", cast_256_LTX__mcrypt_encrypt},
	{"cast_256_LTX__mcrypt_decrypt", cast_256_LTX__mcrypt_decrypt},
	{"cast_256_LTX__mcrypt_get_size", cast_256_LTX__mcrypt_get_size},
	{"cast_256_LTX__mcrypt_get_block_size", cast_256_LTX__mcrypt_get_block_size},
	{"cast_256_LTX__is_block_algorithm", cast_256_LTX__is_block_algorithm},
	{"cast_256_LTX__mcrypt_get_key_size", cast_256_LTX__mcrypt_get_key_size},
	{"cast_256_LTX__mcrypt_get_supported_key_sizes", cast_256_LTX__mcrypt_get_supported_key_sizes},
	{"cast_256_LTX__mcrypt_get_algorithms_name", cast_256_LTX__mcrypt_get_algorithms_name},
	{"cast_256_LTX__mcrypt_self_test", cast_256_LTX__mcrypt_self_test},
	{"cast_256_LTX__mcrypt_algorithm_version", cast_256_LTX__mcrypt_algorithm_version},
	{"loki97", NULL}, 
	{"loki97_LTX__mcrypt_set_key", loki97_LTX__mcrypt_set_key},
	{"loki97_LTX__mcrypt_encrypt", loki97_LTX__mcrypt_encrypt},
	{"loki97_LTX__mcrypt_decrypt", loki97_LTX__mcrypt_decrypt},
	{"loki97_LTX__mcrypt_get_size", loki97_LTX__mcrypt_get_size},
	{"loki97_LTX__mcrypt_get_block_size", loki97_LTX__mcrypt_get_block_size},
	{"loki97_LTX__is_block_algorithm", loki97_LTX__is_block_algorithm},
	{"loki97_LTX__mcrypt_get_key_size", loki97_LTX__mcrypt_get_key_size},
	{"loki97_LTX__mcrypt_get_supported_key_sizes", loki97_LTX__mcrypt_get_supported_key_sizes},
	{"loki97_LTX__mcrypt_get_algorithms_name", loki97_LTX__mcrypt_get_algorithms_name},
	{"loki97_LTX__mcrypt_self_test", loki97_LTX__mcrypt_self_test},
	{"loki97_LTX__mcrypt_algorithm_version", loki97_LTX__mcrypt_algorithm_version},
	{"rijndael-192", NULL}, 
	{"rijndael_192_LTX__mcrypt_set_key", rijndael_192_LTX__mcrypt_set_key},
	{"rijndael_192_LTX__mcrypt_encrypt", rijndael_192_LTX__mcrypt_encrypt},
	{"rijndael_192_LTX__mcrypt_decrypt", rijndael_192_LTX__mcrypt_decrypt},
	{"rijndael_192_LTX__mcrypt_get_size", rijndael_192_LTX__mcrypt_get_size},
	{"rijndael_192_LTX__mcrypt_get_block_size", rijndael_192_LTX__mcrypt_get_block_size},
	{"rijndael_192_LTX__is_block_algorithm", rijndael_192_LTX__is_block_algorithm},
	{"rijndael_192_LTX__mcrypt_get_key_size", rijndael_192_LTX__mcrypt_get_key_size},
	{"rijndael_192_LTX__mcrypt_get_supported_key_sizes", rijndael_192_LTX__mcrypt_get_supported_key_sizes},
	{"rijndael_192_LTX__mcrypt_get_algorithms_name", rijndael_192_LTX__mcrypt_get_algorithms_name},
	{"rijndael_192_LTX__mcrypt_self_test", rijndael_192_LTX__mcrypt_self_test},
	{"rijndael_192_LTX__mcrypt_algorithm_version", rijndael_192_LTX__mcrypt_algorithm_version},
	{"saferplus", NULL}, 
	{"saferplus_LTX__mcrypt_set_key", saferplus_LTX__mcrypt_set_key},
	{"saferplus_LTX__mcrypt_encrypt", saferplus_LTX__mcrypt_encrypt},
	{"saferplus_LTX__mcrypt_decrypt", saferplus_LTX__mcrypt_decrypt},
	{"saferplus_LTX__mcrypt_get_size", saferplus_LTX__mcrypt_get_size},
	{"saferplus_LTX__mcrypt_get_block_size", saferplus_LTX__mcrypt_get_block_size},
	{"saferplus_LTX__is_block_algorithm", saferplus_LTX__is_block_algorithm},
	{"saferplus_LTX__mcrypt_get_key_size", saferplus_LTX__mcrypt_get_key_size},
	{"saferplus_LTX__mcrypt_get_supported_key_sizes", saferplus_LTX__mcrypt_get_supported_key_sizes},
	{"saferplus_LTX__mcrypt_get_algorithms_name", saferplus_LTX__mcrypt_get_algorithms_name},
	{"saferplus_LTX__mcrypt_self_test", saferplus_LTX__mcrypt_self_test},
	{"saferplus_LTX__mcrypt_algorithm_version", saferplus_LTX__mcrypt_algorithm_version},
	{"wake", NULL}, 
	{"wake_LTX__mcrypt_set_key", wake_LTX__mcrypt_set_key},
	{"wake_LTX__mcrypt_encrypt", wake_LTX__mcrypt_encrypt},
	{"wake_LTX__mcrypt_decrypt", wake_LTX__mcrypt_decrypt},
	{"wake_LTX__mcrypt_get_size", wake_LTX__mcrypt_get_size},
	{"wake_LTX__mcrypt_get_block_size", wake_LTX__mcrypt_get_block_size},
	{"wake_LTX__is_block_algorithm", wake_LTX__is_block_algorithm},
	{"wake_LTX__mcrypt_get_key_size", wake_LTX__mcrypt_get_key_size},
	{"wake_LTX__mcrypt_get_algo_iv_size", wake_LTX__mcrypt_get_algo_iv_size},
	{"wake_LTX__mcrypt_get_supported_key_sizes", wake_LTX__mcrypt_get_supported_key_sizes},
	{"wake_LTX__mcrypt_get_algorithms_name", wake_LTX__mcrypt_get_algorithms_name},
	{"wake_LTX__mcrypt_self_test", wake_LTX__mcrypt_self_test},
	{"wake_LTX__mcrypt_algorithm_version", wake_LTX__mcrypt_algorithm_version},
	{"blowfish-compat", NULL}, 
	{"blowfish_compat_LTX__mcrypt_set_key", blowfish_compat_LTX__mcrypt_set_key},
	{"blowfish_compat_LTX__mcrypt_encrypt", blowfish_compat_LTX__mcrypt_encrypt},
	{"blowfish_compat_LTX__mcrypt_decrypt", blowfish_compat_LTX__mcrypt_decrypt},
	{"blowfish_compat_LTX__mcrypt_get_size", blowfish_compat_LTX__mcrypt_get_size},
	{"blowfish_compat_LTX__mcrypt_get_block_size", blowfish_compat_LTX__mcrypt_get_block_size},
	{"blowfish_compat_LTX__is_block_algorithm", blowfish_compat_LTX__is_block_algorithm},
	{"blowfish_compat_LTX__mcrypt_get_key_size", blowfish_compat_LTX__mcrypt_get_key_size},
	{"blowfish_compat_LTX__mcrypt_get_supported_key_sizes", blowfish_compat_LTX__mcrypt_get_supported_key_sizes},
	{"blowfish_compat_LTX__mcrypt_get_algorithms_name", blowfish_compat_LTX__mcrypt_get_algorithms_name},
	{"blowfish_compat_LTX__mcrypt_self_test", blowfish_compat_LTX__mcrypt_self_test},
	{"blowfish_compat_LTX__mcrypt_algorithm_version", blowfish_compat_LTX__mcrypt_algorithm_version},
	{"des", NULL}, 
	{"des_LTX__mcrypt_set_key", des_LTX__mcrypt_set_key},
	{"des_LTX__mcrypt_encrypt", des_LTX__mcrypt_encrypt},
	{"des_LTX__mcrypt_decrypt", des_LTX__mcrypt_decrypt},
	{"des_LTX__mcrypt_get_size", des_LTX__mcrypt_get_size},
	{"des_LTX__mcrypt_get_block_size", des_LTX__mcrypt_get_block_size},
	{"des_LTX__is_block_algorithm", des_LTX__is_block_algorithm},
	{"des_LTX__mcrypt_get_key_size", des_LTX__mcrypt_get_key_size},
	{"des_LTX__mcrypt_get_supported_key_sizes", des_LTX__mcrypt_get_supported_key_sizes},
	{"des_LTX__mcrypt_get_algorithms_name", des_LTX__mcrypt_get_algorithms_name},
	{"des_LTX__mcrypt_self_test", des_LTX__mcrypt_self_test},
	{"des_LTX__mcrypt_algorithm_version", des_LTX__mcrypt_algorithm_version},
	{"rijndael-256", NULL}, 
	{"rijndael_256_LTX__mcrypt_set_key", rijndael_256_LTX__mcrypt_set_key},
	{"rijndael_256_LTX__mcrypt_encrypt", rijndael_256_LTX__mcrypt_encrypt},
	{"rijndael_256_LTX__mcrypt_decrypt", rijndael_256_LTX__mcrypt_decrypt},
	{"rijndael_256_LTX__mcrypt_get_size", rijndael_256_LTX__mcrypt_get_size},
	{"rijndael_256_LTX__mcrypt_get_block_size", rijndael_256_LTX__mcrypt_get_block_size},
	{"rijndael_256_LTX__is_block_algorithm", rijndael_256_LTX__is_block_algorithm},
	{"rijndael_256_LTX__mcrypt_get_key_size", rijndael_256_LTX__mcrypt_get_key_size},
	{"rijndael_256_LTX__mcrypt_get_supported_key_sizes", rijndael_256_LTX__mcrypt_get_supported_key_sizes},
	{"rijndael_256_LTX__mcrypt_get_algorithms_name", rijndael_256_LTX__mcrypt_get_algorithms_name},
	{"rijndael_256_LTX__mcrypt_self_test", rijndael_256_LTX__mcrypt_self_test},
	{"rijndael_256_LTX__mcrypt_algorithm_version", rijndael_256_LTX__mcrypt_algorithm_version},
	{"serpent", NULL}, 
	{"serpent_LTX__mcrypt_set_key", serpent_LTX__mcrypt_set_key},
	{"serpent_LTX__mcrypt_encrypt", serpent_LTX__mcrypt_encrypt},
	{"serpent_LTX__mcrypt_decrypt", serpent_LTX__mcrypt_decrypt},
	{"serpent_LTX__mcrypt_get_size", serpent_LTX__mcrypt_get_size},
	{"serpent_LTX__mcrypt_get_block_size", serpent_LTX__mcrypt_get_block_size},
	{"serpent_LTX__is_block_algorithm", serpent_LTX__is_block_algorithm},
	{"serpent_LTX__mcrypt_get_key_size", serpent_LTX__mcrypt_get_key_size},
	{"serpent_LTX__mcrypt_get_supported_key_sizes", serpent_LTX__mcrypt_get_supported_key_sizes},
	{"serpent_LTX__mcrypt_get_algorithms_name", serpent_LTX__mcrypt_get_algorithms_name},
	{"serpent_LTX__mcrypt_self_test", serpent_LTX__mcrypt_self_test},
	{"serpent_LTX__mcrypt_algorithm_version", serpent_LTX__mcrypt_algorithm_version},
	{"xtea", NULL}, 
	{"xtea_LTX__mcrypt_set_key", xtea_LTX__mcrypt_set_key},
	{"xtea_LTX__mcrypt_encrypt", xtea_LTX__mcrypt_encrypt},
	{"xtea_LTX__mcrypt_decrypt", xtea_LTX__mcrypt_decrypt},
	{"xtea_LTX__mcrypt_get_size", xtea_LTX__mcrypt_get_size},
	{"xtea_LTX__mcrypt_get_block_size", xtea_LTX__mcrypt_get_block_size},
	{"xtea_LTX__is_block_algorithm", xtea_LTX__is_block_algorithm},
	{"xtea_LTX__mcrypt_get_key_size", xtea_LTX__mcrypt_get_key_size},
	{"xtea_LTX__mcrypt_get_supported_key_sizes", xtea_LTX__mcrypt_get_supported_key_sizes},
	{"xtea_LTX__mcrypt_get_algorithms_name", xtea_LTX__mcrypt_get_algorithms_name},
	{"xtea_LTX__mcrypt_self_test", xtea_LTX__mcrypt_self_test},
	{"xtea_LTX__mcrypt_algorithm_version", xtea_LTX__mcrypt_algorithm_version},
	{"blowfish", NULL}, 
	{"blowfish_LTX__mcrypt_set_key", blowfish_LTX__mcrypt_set_key},
	{"blowfish_LTX__mcrypt_encrypt", blowfish_LTX__mcrypt_encrypt},
	{"blowfish_LTX__mcrypt_decrypt", blowfish_LTX__mcrypt_decrypt},
	{"blowfish_LTX__mcrypt_get_size", blowfish_LTX__mcrypt_get_size},
	{"blowfish_LTX__mcrypt_get_block_size", blowfish_LTX__mcrypt_get_block_size},
	{"blowfish_LTX__is_block_algorithm", blowfish_LTX__is_block_algorithm},
	{"blowfish_LTX__mcrypt_get_key_size", blowfish_LTX__mcrypt_get_key_size},
	{"blowfish_LTX__mcrypt_get_supported_key_sizes", blowfish_LTX__mcrypt_get_supported_key_sizes},
	{"blowfish_LTX__mcrypt_get_algorithms_name", blowfish_LTX__mcrypt_get_algorithms_name},
	{"blowfish_LTX__mcrypt_self_test", blowfish_LTX__mcrypt_self_test},
	{"blowfish_LTX__mcrypt_algorithm_version", blowfish_LTX__mcrypt_algorithm_version},
	{"enigma", NULL}, 
	{"enigma_LTX__mcrypt_set_key", enigma_LTX__mcrypt_set_key},
	{"enigma_LTX__mcrypt_encrypt", enigma_LTX__mcrypt_encrypt},
	{"enigma_LTX__mcrypt_decrypt", enigma_LTX__mcrypt_decrypt},
	{"enigma_LTX__mcrypt_get_size", enigma_LTX__mcrypt_get_size},
	{"enigma_LTX__mcrypt_get_block_size", enigma_LTX__mcrypt_get_block_size},
	{"enigma_LTX__is_block_algorithm", enigma_LTX__is_block_algorithm},
	{"enigma_LTX__mcrypt_get_key_size", enigma_LTX__mcrypt_get_key_size},
	{"enigma_LTX__mcrypt_get_algo_iv_size", enigma_LTX__mcrypt_get_algo_iv_size},
	{"enigma_LTX__mcrypt_get_supported_key_sizes", enigma_LTX__mcrypt_get_supported_key_sizes},
	{"enigma_LTX__mcrypt_get_algorithms_name", enigma_LTX__mcrypt_get_algorithms_name},
	{"enigma_LTX__mcrypt_self_test", enigma_LTX__mcrypt_self_test},
	{"enigma_LTX__mcrypt_algorithm_version", enigma_LTX__mcrypt_algorithm_version},
	{"rc2", NULL}, 
	{"rc2_LTX__mcrypt_set_key", rc2_LTX__mcrypt_set_key},
	{"rc2_LTX__mcrypt_encrypt", rc2_LTX__mcrypt_encrypt},
	{"rc2_LTX__mcrypt_decrypt", rc2_LTX__mcrypt_decrypt},
	{"rc2_LTX__mcrypt_get_size", rc2_LTX__mcrypt_get_size},
	{"rc2_LTX__mcrypt_get_block_size", rc2_LTX__mcrypt_get_block_size},
	{"rc2_LTX__is_block_algorithm", rc2_LTX__is_block_algorithm},
	{"rc2_LTX__mcrypt_get_key_size", rc2_LTX__mcrypt_get_key_size},
	{"rc2_LTX__mcrypt_get_supported_key_sizes", rc2_LTX__mcrypt_get_supported_key_sizes},
	{"rc2_LTX__mcrypt_get_algorithms_name", rc2_LTX__mcrypt_get_algorithms_name},
	{"rc2_LTX__mcrypt_self_test", rc2_LTX__mcrypt_self_test},
	{"rc2_LTX__mcrypt_algorithm_version", rc2_LTX__mcrypt_algorithm_version},
	{"tripledes", NULL}, 
	{"tripledes_LTX__mcrypt_set_key", tripledes_LTX__mcrypt_set_key},
	{"tripledes_LTX__mcrypt_encrypt", tripledes_LTX__mcrypt_encrypt},
	{"tripledes_LTX__mcrypt_decrypt", tripledes_LTX__mcrypt_decrypt},
	{"tripledes_LTX__mcrypt_get_size", tripledes_LTX__mcrypt_get_size},
	{"tripledes_LTX__mcrypt_get_block_size", tripledes_LTX__mcrypt_get_block_size},
	{"tripledes_LTX__is_block_algorithm", tripledes_LTX__is_block_algorithm},
	{"tripledes_LTX__mcrypt_get_key_size", tripledes_LTX__mcrypt_get_key_size},
	{"tripledes_LTX__mcrypt_get_supported_key_sizes", tripledes_LTX__mcrypt_get_supported_key_sizes},
	{"tripledes_LTX__mcrypt_get_algorithms_name", tripledes_LTX__mcrypt_get_algorithms_name},
	{"tripledes_LTX__mcrypt_self_test", tripledes_LTX__mcrypt_self_test},
	{"tripledes_LTX__mcrypt_algorithm_version", tripledes_LTX__mcrypt_algorithm_version},
	{NULL, NULL}
};
